require 'puppetdb'
require 'train-puppetdb/platform'

module TrainPlugins
	module PuppetDBInterface
		class Connection < Train::Plugins::Transport::BaseConnection
			include TrainPlugins::PuppetDBInterface::Platform

			def initialize(options)
				if options.nil? || options[:server].nil?
					raise Train::TransportError, "You must set a PuppetDB server."
				end

				super(options)

				@tmp_dir = Dir.mktmpdir

				# Check if target is reachable
				begin
					@client = PuppetDB::Client.new({
						:server => options[:server],
						:pem => {
							'key' => sanitize_cert("key", options[:key]),
							'cert' => sanitize_cert("cert", options[:cert]),
							'ca_file' => sanitize_cert("ca", options[:cacert]),
						}
					})

					@client.request('nodes', [:'~', 'certname', '.*'], {:limit => 1})
				rescue => e
					raise Train::TransportError, e.response
				end
			end

			def local?
				false
			end

			def uri
				'puppetdb://'
			end

			def puppetdb_client
				@client
			end

			private

			def sanitize_cert(filename, cert)
				if cert.start_with?("-----")
					File.write("#{@tmp_dir}/#{filename}", cert)
					return "#{@tmp_dir}/#{filename}"
				else
					return cert
				end
			end
		end
	end
end
